@@ -31,6 +31,7 @@
 #include <gdk/gdkx.h>
 #include <X11/Xlib.h>
 #include <glib/gi18n.h>
+#include <gconf/gconf-client.h>
 
 typedef struct App App;
 typedef struct GrabInfo GrabInfo;
@@ -49,9 +50,11 @@ struct App
     GtkWidget	   *panel_checkbox;
     GtkWidget	   *panel_label;
     GtkWidget	   *clone_checkbox;
+    GtkWidget	   *show_icon_checkbox;
     
     GtkWidget      *area;
     gboolean	    ignore_gui_changes;
+    GConfClient	   *client;
 };
 
 static void rebuild_gui (App *app);
@@ -1347,6 +1350,7 @@ paint_output (App *app, cairo_t *cr, int i)
     PangoLayout *layout = get_display_name (app, output);
     PangoRectangle extent;
     GdkRectangle viewport;
+    double angle;
 
     cairo_save (cr);
     
@@ -1373,6 +1377,46 @@ paint_output (App *app, cairo_t *cr, int i)
     g_print ("%f %f %f %f\n", x, y, w * scale + 0.5, h * scale + 0.5);
 #endif
 
+    cairo_save (cr);
+
+    cairo_translate (cr,
+		     x + (w * scale + 0.5) / 2,
+		     y + (h * scale + 0.5) / 2);
+    
+    if (output->rotation & GNOME_RR_ROTATION_0)
+    {
+	angle = 0;
+    }
+    else if (output->rotation & GNOME_RR_ROTATION_90)
+    {
+	angle = G_PI / 2;
+    }
+    else if (output->rotation & GNOME_RR_ROTATION_180)
+    {
+	angle = G_PI;
+    }
+    else if (output->rotation & GNOME_RR_ROTATION_270)
+    {
+	angle = 1.5 * G_PI;
+    }
+    else
+    {
+	angle = 0;
+    }
+
+    cairo_rotate (cr, angle);
+    
+    if (output->rotation & GNOME_RR_REFLECT_X)
+	cairo_scale (cr, -1, 1);
+    
+    if (output->rotation & GNOME_RR_REFLECT_Y)
+	cairo_scale (cr, 1, -1);
+    
+    cairo_translate (cr,	
+		     - x - (w * scale + 0.5) / 2,
+		     - y - (h * scale + 0.5) / 2);
+    
+
     cairo_rectangle (cr, x, y, w * scale + 0.5, h * scale + 0.5);
     cairo_clip_preserve (cr);
     
@@ -1388,12 +1432,12 @@ paint_output (App *app, cairo_t *cr, int i)
     if (output == app->current_output)
     {
 	cairo_rectangle (cr, x + 2, y + 2, w * scale + 0.5 - 4, h * scale + 0.5 - 4);
-    
+	
 	cairo_set_line_width (cr, 4);
 	cairo_set_source_rgba (cr, 0.33, 0.43, 0.57, 1.0);
 	cairo_stroke (cr);
     }
-
+    
     cairo_rectangle (cr, x + 0.5, y + 0.5, w * scale + 0.5 - 1, h * scale + 0.5 - 1);
     
     cairo_set_line_width (cr, 1);
@@ -1417,9 +1461,10 @@ paint_output (App *app, cairo_t *cr, int i)
 	cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
     
     pango_cairo_show_layout (cr, layout);
-    g_object_unref (layout);
 
     cairo_restore (cr);
+    
+    g_object_unref (layout);
 }
 
 static void
@@ -1615,6 +1660,19 @@ on_detect_displays (GtkWidget *widget, gpointer data)
     gnome_rr_screen_refresh (app->screen);
 }
 
+#define SHOW_ICON_KEY "/apps/gnome_settings_daemon/xrandr/show_notification_icon"
+
+
+static void
+on_show_icon_toggled (GtkWidget *widget, gpointer data)
+{
+    GtkToggleButton *tb = GTK_TOGGLE_BUTTON (widget);
+    App *app = data;
+    
+    gconf_client_set_bool (app->client, SHOW_ICON_KEY,
+			   gtk_toggle_button_get_active (tb), NULL);
+}
+
 static void
 run_application (App *app)
 {
@@ -1631,7 +1689,8 @@ run_application (App *app)
 	g_warning ("Could not open " GLADE_FILE);
 	return;
     }
-    
+
+    app->client = gconf_client_get_default ();
     app->screen = gnome_rr_screen_new (gdk_screen_get_default(),
 				 on_screen_changed, app);
 
@@ -1659,6 +1718,13 @@ run_application (App *app)
 
     g_signal_connect (glade_xml_get_widget (xml, "detect_displays_button"),
 		      "clicked", G_CALLBACK (on_detect_displays), app);
+
+    app->show_icon_checkbox = glade_xml_get_widget (xml, "show_notification_icon");
+
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (app->show_icon_checkbox),
+				  gconf_client_get_bool (app->client, SHOW_ICON_KEY, NULL));
+    
+    g_signal_connect (app->show_icon_checkbox, "toggled", G_CALLBACK (on_show_icon_toggled), app);
     
     app->panel_checkbox = glade_xml_get_widget (xml, "panel_checkbox");
     app->panel_label = glade_xml_get_widget (xml, "panel_label");
