@@ -35,7 +35,6 @@
 
 #include "capplet-util.h"
 #include "gconf-property-editor.h"
-#include "libsounds/sound-view.h"
 
 #include <glade/glade.h>
 
@@ -53,8 +52,8 @@
 
 #include "activate-settings-daemon.h"
 #include "pipeline-tests.h"
-
 #include "mixer-support.h"
+#include "sound-theme.h"
 
 typedef enum {
 	AUDIO_PLAYBACK,
@@ -63,6 +62,11 @@ typedef enum {
 	VIDEO_CAPTURE
 } device_type;
 
+typedef enum {
+	SYSTEM_ALSA,
+	SYSTEM_OSS
+} SoundSystemType;
+
 typedef struct _DeviceChooser
 {
 	const gchar *profile;
@@ -72,13 +76,11 @@ typedef struct _DeviceChooser
 	const gchar *test_pipeline;
 } DeviceChooser;
 
-#define ENABLE_ESD_KEY             "/desktop/gnome/sound/enable_esd"
 #define EVENT_SOUNDS_KEY           "/desktop/gnome/sound/event_sounds"
+#define INPUT_SOUNDS_KEY           "/desktop/gnome/sound/input_feedback_sounds"
 #define DEFAULT_MIXER_DEVICE_KEY   "/desktop/gnome/sound/default_mixer_device"
 #define DEFAULT_MIXER_TRACKS_KEY   "/desktop/gnome/sound/default_mixer_tracks"
-#define VISUAL_BELL_KEY            "/apps/metacity/general/visual_bell"
 #define AUDIO_BELL_KEY             "/apps/metacity/general/audible_bell"
-#define VISUAL_BELL_TYPE_KEY       "/apps/metacity/general/visual_bell_type"
 #define GST_GCONF_DIR              "/system/gstreamer/0.10"
 
 #define AUDIO_TEST_SOURCE          "audiotestsrc wave=sine freq=512"
@@ -88,7 +90,6 @@ typedef struct _DeviceChooser
 
 /* Capplet-specific prototypes */
 
-static SoundProperties *props = NULL;
 static GConfClient     *gconf_client = NULL;
 static GtkWidget       *dialog_win = NULL;
 static GList           *device_choosers = NULL;
@@ -111,78 +112,18 @@ CheckXKB (void)
 	return have_xkb;
 }
 
-static void
-props_changed_cb (SoundProperties *p, SoundEvent *event, gpointer data)
-{
-	sound_properties_user_save (p);
-}
-
-
-
-static GConfEnumStringPair bell_flash_enums[] = {
-	{ 0, "frame_flash" },
-	{ 1, "fullscreen" },
-	{ -1, NULL }
-};
-
-static GConfValue *
-bell_flash_from_widget (GConfPropertyEditor *peditor, const GConfValue *value)
-{
-	GConfValue *new_value;
-
-	new_value = gconf_value_new (GCONF_VALUE_STRING);
-	gconf_value_set_string (new_value,
-				gconf_enum_to_string (bell_flash_enums, gconf_value_get_int (value)));
-
-	return new_value;
-}
-
-static GConfValue *
-bell_flash_to_widget (GConfPropertyEditor *peditor, const GConfValue *value)
-{
-	GConfValue *new_value;
-	const gchar *str;
-	gint val = 2;
-
-	str = (value && (value->type == GCONF_VALUE_STRING)) ? gconf_value_get_string (value) : NULL;
-
-	new_value = gconf_value_new (GCONF_VALUE_INT);
-	if (value->type == GCONF_VALUE_STRING) {
-		gconf_string_to_enum (bell_flash_enums,
-				      str,
-				      &val);
-	}
-	gconf_value_set_int (new_value, val);
-
-	return new_value;
-}
-
 static GladeXML *
 create_dialog (void)
 {
 	GladeXML *dialog;
-	GtkWidget *widget, *box, *view, *image;
+	GtkWidget *image;
 
-	dialog = glade_xml_new (GNOMECC_GLADE_DIR "/sound-properties.glade", "sound_prefs_dialog", NULL);
-	if (dialog == NULL)
-	  return NULL;
-
-	widget = glade_xml_get_widget (dialog, "sound_prefs_dialog");
-
-	props = sound_properties_new ();
-	sound_properties_add_defaults (props, NULL);
-	g_signal_connect (G_OBJECT (props), "event_changed",
-			  (GCallback) props_changed_cb, NULL);
-	view = sound_view_new (props);
-	box = glade_xml_get_widget (dialog, "events_vbox");
-	gtk_box_pack_start (GTK_BOX (box), view, TRUE, TRUE, 0);
-	gtk_widget_show_all (view);
-
-	g_signal_connect_swapped (G_OBJECT (widget), "destroy",
-				  (GCallback) gtk_object_destroy, props);
-
-	gtk_image_set_from_file (GTK_IMAGE (WID ("bell_image")),
-				 GNOMECC_DATA_DIR "/pixmaps/visual-bell.png");
+	dialog = glade_xml_new ("sound-properties.glade", "sound_prefs_dialog", NULL);
+	if (dialog == NULL) {
+		dialog = glade_xml_new (GNOMECC_GLADE_DIR "/sound-properties.glade", "sound_prefs_dialog", NULL);
+		if (dialog == NULL)
+			return NULL;
+	}
 
 	image = gtk_image_new_from_stock (GTK_STOCK_APPLY, GTK_ICON_SIZE_BUTTON);
 	gtk_button_set_image (GTK_BUTTON (WID ("sounds_playback_test")), image);
@@ -196,10 +137,6 @@ create_dialog (void)
 	image = gtk_image_new_from_stock (GTK_STOCK_APPLY, GTK_ICON_SIZE_BUTTON);
 	gtk_button_set_image (GTK_BUTTON (WID ("chat_audio_capture_test")), image);
 
-	if (!CheckXKB()) {
-		gtk_widget_set_sensitive (WID ("bell_flash_alignment"), FALSE);
-	}
-
 	return dialog;
 }
 
@@ -466,15 +403,38 @@ remove_device (int type, const gchar *pipeline)
 	}
 }
 
-static void
-device_added_alsa (LibHalContext *ctx, const char *udi)
+static gboolean
+filter_device_generic (LibHalContext *ctx, const char *udi)
+{
+	gboolean ignore = FALSE;
+	gchar *parent_udi;
+	gchar *subsystem;
+
+	parent_udi = libhal_device_get_property_string (ctx, udi, "info.parent", NULL);
+
+	if (!parent_udi)
+		return FALSE;
+
+	subsystem = libhal_device_get_property_string (ctx, parent_udi, "info.subsystem", NULL);
+
+	/* filter out pc speaker */
+	if (subsystem && (!strcmp (subsystem, "platform") || !strcmp (subsystem, "sound"))) {
+		gchar *device_id = libhal_device_get_property_string (ctx, parent_udi, "platform.id", NULL);
+		ignore = device_id && !strncmp (device_id, "pcspk", 5);
+		libhal_free_string (device_id);
+	}
+
+	libhal_free_string (parent_udi);
+	libhal_free_string (subsystem);
+
+	return ignore;
+}
+
+static gboolean
+filter_device_alsa (LibHalContext *ctx, const char *udi)
 {
-	gchar *type_string;
-	gchar *class_string;
-	int type;
-	const gchar *element;
-	gchar *pipeline, *description;
 	gboolean ignore;
+	gchar *class_string;
 
 	/* filter out "digitizer", "modem", "none", "unknown" */
 	class_string = libhal_device_get_property_string (ctx, udi, "alsa.pcm_class", NULL);
@@ -482,10 +442,94 @@ device_added_alsa (LibHalContext *ctx, const char *udi)
 		 && strcmp (class_string, "generic") != 0
 		 && strcmp (class_string, "multi") != 0;
 	libhal_free_string (class_string);
-	if (ignore) {
-		return;
+
+	if (!ignore)
+		ignore = filter_device_generic (ctx, udi);
+
+	return ignore;
+}
+
+static gboolean
+filter_device_oss (LibHalContext *ctx, const char *udi)
+{
+	gboolean ignore = FALSE;
+	gchar *parent_udi;
+	gchar *subsystem;
+
+	parent_udi = libhal_device_get_property_string (ctx, udi, "info.parent", NULL);
+
+	if (!parent_udi)
+		return FALSE;
+
+	subsystem = libhal_device_get_property_string (ctx, parent_udi, "info.subsystem", NULL);
+	/* filter out modem devices */
+	if (subsystem && !strcmp (subsystem, "pci")) {
+		dbus_int32_t device_class = libhal_device_get_property_int (ctx, parent_udi, "pci.device_class", NULL);
+		/* this means "Simple communications controllers". Maybe there is a headerfile with definitions?
+		 * visit http://www.acm.uiuc.edu/sigops/roll_your_own/7.c.1.html for further information */
+		ignore = (device_class == 0x7);
 	}
 
+	libhal_free_string (parent_udi);
+	libhal_free_string (subsystem);
+
+	if (!ignore)
+		ignore = filter_device_generic (ctx, udi);
+
+	return ignore;
+}
+
+static gchar *
+get_device_description (LibHalContext *ctx, const char *udi, SoundSystemType snd_sys)
+{
+	gchar *card_id = NULL, *device_id = NULL, *product, *desc;
+	const gchar *indicator = NULL;
+
+	/* the device number should be reported if there is a second (playback)
+	 * device with the same description on the same card */
+	/* dbus_int32_t device_num = 0; */
+
+	switch (snd_sys) {
+	case SYSTEM_ALSA:
+	       	card_id = libhal_device_get_property_string (ctx, udi, "alsa.card_id", NULL);
+		device_id = libhal_device_get_property_string (ctx, udi, "alsa.device_id", NULL);
+		/* device_num = libhal_device_get_property_int (ctx, udi, "alsa.device", NULL); */
+		indicator = "(ALSA)";
+		break;
+	case SYSTEM_OSS:
+		card_id = libhal_device_get_property_string (ctx, udi, "oss.card_id", NULL);
+		device_id = libhal_device_get_property_string (ctx, udi, "oss.device_id", NULL);
+		/* device_num = libhal_device_get_property_int (ctx, udi, "oss.device", NULL); */
+		indicator = "(OSS)";
+	}
+
+	/* card_id and device_id are not mandatory in OSS and ALSA namespace
+	 * according to the specification so it could be possible go get an
+	 * empty string */
+	if (card_id && device_id) {
+		desc = g_strconcat (card_id, " ", device_id, " ", indicator, NULL);
+	} else {
+		product = libhal_device_get_property_string (ctx, udi, "info.product", NULL);
+		desc = g_strconcat (product, " ", indicator, NULL);
+		libhal_free_string (product);
+	}
+	libhal_free_string (card_id);
+	libhal_free_string (device_id);
+
+	return desc;
+}
+
+static void
+device_added_alsa (LibHalContext *ctx, const char *udi)
+{
+	gchar *type_string;
+	int type;
+	const gchar *element;
+	gchar *pipeline, *description;
+
+	if (filter_device_alsa (ctx, udi))
+		return;
+
 	type_string = libhal_device_get_property_string (ctx, udi, "alsa.type", NULL);
 	if (strcmp (type_string, "playback") == 0) {
 		type = AUDIO_PLAYBACK;
@@ -503,12 +547,12 @@ device_added_alsa (LibHalContext *ctx, const char *udi)
 	}
 
 	pipeline = g_strconcat (element, " udi=", udi, NULL);
-	description = libhal_device_get_property_string (ctx, udi, "alsa.device_id", NULL);
+	description = get_device_description (ctx, udi, SYSTEM_ALSA);
 
 	add_device (type, pipeline, description, NULL);
 
 	g_free (pipeline);
-	libhal_free_string (description);
+	g_free (description);
 }
 
 static void
@@ -519,6 +563,9 @@ device_added_oss (LibHalContext *ctx, const char *udi)
 	const gchar *element;
 	gchar *pipeline, *description;
 
+	if (filter_device_oss (ctx, udi))
+		return;
+
 	type_string = libhal_device_get_property_string (ctx, udi, "oss.type", NULL);
 	if (strcmp (type_string, "pcm") == 0) {
 		type = AUDIO_PLAYBACK;
@@ -536,12 +583,12 @@ device_added_oss (LibHalContext *ctx, const char *udi)
 	}
 
 	pipeline = g_strconcat (element, " udi=", udi, NULL);
-	description = libhal_device_get_property_string (ctx, udi, "oss.device_id", NULL);
+	description = get_device_description (ctx, udi, SYSTEM_OSS);
 
 	add_device (type, pipeline, description, NULL);
 
 	g_free (pipeline);
-	libhal_free_string (description);
+	g_free (description);
 }
 
 static void
@@ -1082,29 +1129,16 @@ setup_dialog (GladeXML *dialog, GConfChangeSet *changeset)
 			      WID ("chat_audio_capture_test"),
 			      "gconfaudiosrc" AUDIO_TEST_IN_BETWEEN "gconfaudiosink profile=chat");
 
-#ifdef HAVE_ESD
-	peditor = gconf_peditor_new_boolean (NULL, ENABLE_ESD_KEY, WID ("enable_toggle"), NULL);
-	gconf_peditor_widget_set_guard (GCONF_PROPERTY_EDITOR (peditor), WID ("events_toggle"));
-	gconf_peditor_widget_set_guard (GCONF_PROPERTY_EDITOR (peditor), WID ("events_vbox"));
-#else
-	gtk_widget_hide (WID ("enable_toggle"));
-#endif
-	gconf_peditor_new_boolean (NULL, EVENT_SOUNDS_KEY, WID ("events_toggle"), NULL);
-
+	peditor = gconf_peditor_new_boolean (NULL, EVENT_SOUNDS_KEY, WID ("events_toggle"), NULL);
+	gconf_peditor_widget_set_guard (GCONF_PROPERTY_EDITOR (peditor), WID ("input_feedback_toggle"));
+	gconf_peditor_widget_set_guard (GCONF_PROPERTY_EDITOR (peditor), WID ("sound_theme_combobox"));
+	gconf_peditor_widget_set_guard (GCONF_PROPERTY_EDITOR (peditor), WID ("sounds_treeview"));
+	gconf_peditor_new_boolean (NULL, INPUT_SOUNDS_KEY, WID ("input_feedback_toggle"), NULL);
 	gconf_peditor_new_boolean (NULL, AUDIO_BELL_KEY, WID ("bell_audible_toggle"), NULL);
 
-	peditor = gconf_peditor_new_boolean (NULL, VISUAL_BELL_KEY, WID ("bell_visual_toggle"), NULL);
-	gconf_peditor_widget_set_guard (GCONF_PROPERTY_EDITOR (peditor), WID ("bell_flash_vbox"));
-
-	/* peditor not so convenient for the radiobuttons */
-	gconf_peditor_new_select_radio (NULL,
-					VISUAL_BELL_TYPE_KEY,
-					gtk_radio_button_get_group (GTK_RADIO_BUTTON (WID ("bell_flash_window_radio"))),
-					"conv-to-widget-cb", bell_flash_to_widget,
-					"conv-from-widget-cb", bell_flash_from_widget,
-					NULL);
-
 	setup_default_mixer (dialog);
+	setup_sound_theme (dialog);
+	setup_sound_theme_custom (dialog, CheckXKB());
 }
 
 /* get_legacy_settings
@@ -1123,7 +1157,6 @@ get_legacy_settings (void)
 	gboolean val_bool, def;
 
 	client = gconf_client_get_default ();
-	COPY_FROM_LEGACY (bool, "/desktop/gnome/sound/enable_esd", "/sound/system/settings/start_esd=false");
 	COPY_FROM_LEGACY (bool, "/desktop/gnome/sound/event_sounds", "/sound/system/settings/event_sounds=false");
 	g_object_unref (G_OBJECT (client));
 }
